/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * phyio - small tool for ethernet phy register access
 *
 * Copyright (C) 2013-2020 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
 * Author: Markus Niebel
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <linux/sockios.h>
#include <linux/if.h>
#include <linux/mii.h>
#include <linux/mdio.h>

#define DEFAULT_ETH_NAME "eth0"

int read_phy(int fd, struct ifreq *ifr)
{
	int err;
	
	err = ioctl(fd, SIOCGMIIREG, ifr);
	if (err)
		perror("ioctl SIOCGMIIREG");

	return err;
}

int write_phy(int fd, struct ifreq *ifr)
{
	int err;
	
	err = ioctl(fd, SIOCSMIIREG, ifr);
	if (err)
		perror("ioctl SIOCSMIIREG");

	return err;
}

void print_help(const char *prgn)
{
	fprintf(stderr,"%s [-d dev] -a phy_addr -r reg [-w val] \n",prgn);
	fprintf(stderr,"\tdev: device, e.g. %s    (def)\n", DEFAULT_ETH_NAME);
	fprintf(stderr,"\tphy_addr: phy id\n");
	fprintf(stderr,"\treg: phy register address\n");
	fprintf(stderr,"\tval : register value to write to phy (default is reading!)\n");
}

int main(int argc, char **argv)
{
	int i;
	uint32_t val;
	char *dn = DEFAULT_ETH_NAME;
	unsigned int addr = 0xffff;
	unsigned int reg = 0xffffffff;
	int rwmode = 0;
	int fd;
	struct ifreq ifr;
	struct mii_ioctl_data* mii = (struct mii_ioctl_data*)(&ifr.ifr_data);

	if (argc < 2) {
		print_help(argv[0]);
		exit(1);
	}

	while((i = getopt(argc, argv, "d:a:r:w:h")) >= 0) {
		switch(i){
		case 'h':
		    print_help(argv[0]);
		    exit(1);
		    break;
		case 'd':
		    dn = optarg;
		    break;
		case 'a':
		    if(sscanf(optarg,"0x%x",&addr)!=1) {
			fprintf(stderr,"Cannot parse '%s' as addrs., example: 0x0a\n",
				optarg);
			exit(1);
		    }
		    break;
		case 'r':
		    if(sscanf(optarg,"0x%x",&reg)!=1) {
			fprintf(stderr,"Cannot parse '%s' as register, example: 0x1234\n",
				optarg);
			exit(1);
		    }
		    break;
		case 'w':
		    rwmode++;
		    if(sscanf(optarg,"0x%x",&val)!=1) {
			fprintf(stderr,"Cannot parse '%s' as value, example: 0x12345678\n",
				optarg);
			exit(1);
		    }
		    break;
		}
	}

	if((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("error creating socket");
		exit(errno);
	}

	/***
	*** I'm not the one to blame if you screw your phy!
	***/

	memset(&ifr, 0, sizeof(ifr));
	strcpy(ifr.ifr_name, dn); //set to whatever your ethernet device is
	mii->phy_id = addr; //set to your phy's ID
	mii->reg_num = reg; //the register you want to read
	mii->val_in = 0;
	mii->val_out = 0;

	if(rwmode) {
		fprintf(stdout,"value     : 0x%08x\n", val);
		mii->val_in = val;
		if(write_phy(fd, &ifr) < 0)
			exit(errno);
	} else {
		if(read_phy(fd, &ifr) < 0)
			exit(errno);
		fprintf(stdout, "%#08x\n", mii->val_out);
	}

	close(fd);
	exit(0);

}
