/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * A tool to access SMSC9303 switch via I2C
 *
 * Copyright (C) 2013-2020 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
 * Author: Markus Niebel
 *
 * inspired by i2c tools
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <sys/ioctl.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>

#define DEFAULT_I2C_BUS		"/dev/i2c-0"
#define DEFAULT_SMSC9303_ADDR	0x0a         /* SMSC9303 def. i2c addr */

/* return 0 on success, -1 on failure */
static int smsc9303_write(int fd, unsigned int addr, unsigned int reg,
			uint32_t data)
{
	struct i2c_rdwr_ioctl_data msg_rdwr;
	struct i2c_msg             i2cmsg;
	int i;
	__u8 wbuf[4 + 1];

	if((reg / 4) > 256){
	    fprintf(stderr,"Sorry, reg(%d) out of range\n", reg);
	    return -1;
	}
	if(reg % 4){
	    fprintf(stderr,"Sorry, reg(%d) misaligned\n", reg);
	    return -1;
	}

	wbuf[0] = (char)(reg >> 2);  /* _buf[0] is register pointer */
	for(i = 0; i < 4; ++i) /* copy buf[0..n] -> _buf[1..n+1] */
	    wbuf[1 + i] = data >> (24 - (i * 8));

	msg_rdwr.msgs = &i2cmsg;
	msg_rdwr.nmsgs = 1;

	i2cmsg.addr  = addr;
	i2cmsg.flags = 0;
	i2cmsg.len   = sizeof(wbuf);
	i2cmsg.buf   = wbuf;

	if((i=ioctl(fd, I2C_RDWR, &msg_rdwr)) < 0){
	    perror("ioctl()");
	    fprintf(stderr,"ioctl returned %d\n",i);
	    return -1;
	}

	fprintf(stderr,"Wrote %08x to smsc9393 at 0x%02x, reg %08x\n",
		    data, addr, reg);

	return 0;
}

/* return -1 on error, 0 on success */
int smsc9303_read(int fd, unsigned int addr, unsigned int reg,
		uint32_t *data)
{
	struct i2c_rdwr_ioctl_data msg_rdwr;
	struct i2c_msg             i2cmsg[2];
	int i;
	__u8 wbuf[1];
	__u8 rbuf[4];

	if((reg / 4) > 256){
	    fprintf(stderr,"Sorry, reg(%d) out of range\n", addr);
	    return -1;
	}
	if(reg % 4){
	    fprintf(stderr,"Sorry, reg(%d) misaligned\n", addr);
	    return -1;
	}

	msg_rdwr.msgs  = i2cmsg;
	msg_rdwr.nmsgs = 2;

	wbuf[0] = (char)(reg >> 2);  /* wbuf[0] is register pointer */
	i2cmsg[0].addr  = addr;
	i2cmsg[0].flags = 0;
	i2cmsg[0].len   = 1;
	i2cmsg[0].buf   = wbuf;

	i2cmsg[1].addr  = addr;
	i2cmsg[1].flags = I2C_M_RD;
	i2cmsg[1].len   = 4;
	i2cmsg[1].buf   = rbuf;

	if((i = ioctl(fd, I2C_RDWR, &msg_rdwr)) < 0){
	    perror("ioctl()");
	    fprintf(stderr,"ioctl returned %d\n",i);
	    return -1;
	}

	*data = 0;
	for(i = 0; i < 4; ++i) /* copy buf[0..n] -> _buf[1..n+1] */
	    *data |= (((uint32_t)rbuf[i]) << (24 - (i * 8)));

	fprintf(stdout,"Read %08x from smsc9393 at 0x%02x, reg %08x\n",
		    *data, addr, reg);

	return 0;
}



int main(int argc, char **argv)
{
	int i;
	uint32_t val;
	unsigned int reg = 0x50u;
	/* filedescriptor and name of device */
	int d; 
	char *dn=DEFAULT_I2C_BUS;
	unsigned int addr=DEFAULT_SMSC9303_ADDR;
	int rwmode=0;
	while((i=getopt(argc,argv,"d:a:r:w:h"))>=0){
		switch(i){
		case 'h':
		    fprintf(stderr,"%s [-d dev] [-a addr] [-r reg] [-w val] \n",argv[0]);
		    fprintf(stderr,"\tdev: device, e.g. %s    (def)\n", DEFAULT_I2C_BUS);
		    fprintf(stderr,"\taddr: base address of smsc9303, eg 0x%02x (def)\n", DEFAULT_SMSC9303_ADDR);
		    fprintf(stderr,"\treg: full register address (default is 0x50)\n");
		    fprintf(stderr,"\tval : write val to smsc9303 (default is reading!)\n");
		    exit(1);
		    break;
		case 'd':
		    dn=optarg;
		    break;
		case 'a':
		    if(sscanf(optarg,"0x%x",&addr)!=1){
			fprintf(stderr,"Cannot parse '%s' as addrs., example: 0x0a\n",
				optarg);
			exit(1);
		    }
		    break;
		case 'r':
		    if(sscanf(optarg,"0x%x",&reg)!=1){
			fprintf(stderr,"Cannot parse '%s' as register, example: 0x1234\n",
				optarg);
			exit(1);
		    }
		    break;
		case 'w':
		    rwmode++;
		    if(sscanf(optarg,"0x%x",&val)!=1){
			fprintf(stderr,"Cannot parse '%s' as value, example: 0x12345678\n",
				optarg);
			exit(1);
		    }
		    break;
		}
	}

	fprintf(stderr,"address of smsc9303       : 0x%02x\n",addr);
	fprintf(stderr,"register                  : 0x%04x\n", reg);

	if((d = open(dn, O_RDWR)) < 0){
		fprintf(stderr,"Could not open i2c at %s\n",dn);
		perror(dn);
		exit(1);
	}

	fprintf(stderr,"i2c-devicenode is             : %s\n",dn);
	fprintf(stderr,"            on filedescriptor : %d\n\n",d);

	/***
	*** I'm not the one to blame of you screw your switch!
	***/

	if(rwmode){
		if(smsc9303_write(d, addr, reg, val) < 0)
		    exit(1);
	} else {
		if(smsc9303_read(d, addr, reg, &val) < 0)
		    exit(1);
	}


	if(!rwmode){
		fprintf(stdout,"%#08x\n", val);
		close(d);
		exit(1);
	}

	close(d);
	exit(0);

}
