#!/bin/sh
#
# File:        imx-gpio.sh
# Copyright (C) 2015 - 2017 TQ Systems GmbH
# @author Markus Niebel <Markus.Niebel@tq-group.com>
#
# Description: set of shell functions for gpio accesss
#
# License:     GPLv2
#
###############################################################################
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
###############################################################################

readonly SYSFS_GPIO=/sys/class/gpio

#
# calc gpio number
# input: bank (1 ... n)
# input: pin (0 ... 31)
#
imx_gpio() {
	local bank=$1
	local pin=$2
	local gpio="invalid"
	local ret=2

	if [ "${#}" -ne "2" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi

	if [ "${bank}" -eq "${bank}" ] 2>/dev/null; then
		if [ ${bank} -gt 0 ]; then
			if [ "${pin}" -eq "${pin}" ] 2>/dev/null; then
				if [ ${pin} -lt 32 ]; then
					if [ ${pin} -ge 0 ]; then
						gpio=`expr ${bank} - 1`;
						gpio=`expr ${gpio} \* 32`;
						gpio=`expr ${gpio} + ${pin}`;
						echo ${gpio};
						ret=0;
					fi;
				fi;
			fi;
		fi
	fi;

	if [ "$ret" -ne "0" ]; then
		echo "invalid gpio ${bank} ${pin}";
	fi;
	return $ret
}


gpio_export() {
	if [ "${#}" -ne "1" ]
	then
		echo "gpio_export: invalid parameter count ${#}";
		return 1;
	fi
	local gpio=$1
	if [ -d ${SYSFS_GPIO}/gpio${gpio} ]; then
		return 0;
	fi;
	echo ${gpio} > ${SYSFS_GPIO}/export
	if [ -d ${SYSFS_GPIO}/gpio${gpio} ]; then
		return 0;
	fi;
	return 2
}

#
# export one gpio
#
imx_gpio_export() {

	if [ "${#}" -ne "2" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$(imx_gpio $1 $2)
	local ret=$?
	if [ $ret -ne 0 ]; then
		return $ret;
	fi;
	gpio_export ${gpio}
	return $?
}

gpio_dir() {
	if [ "${#}" -ne "1" ]
	then
		echo "_gpio_dir: invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$1
	local dirfile=${SYSFS_GPIO}/gpio${gpio}/direction
	local pindir="unkown"

	if [ $ret -ne 0 ]; then
		return $ret;
	fi;
	if ! [ -r $dirfile ]; then
		echo "gpio ${gpio} not exported"
		return 0;
	fi;

	pindir=$(cat $dirfile)
	if [ $ret -eq 0 ]; then
		echo ${pin}dir;
	fi;
	return $ret
}

imx_gpio_dir() {
	if [ "${#}" -ne "2" ]
	then
		echo "$0: invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$(imx_gpio $1 $2)
	local ret=$?
	if [ $ret -ne 0 ]; then
		return $ret;
	fi;
	gpio_dir ${gpio}
	return $?
}

#
# export one gpio and set it direction
#
gpio_config_dir() {
	if [ "${#}" -ne "2" ]
	then
		echo "gpio_config_dir: invalid parameter count ${#}";
		return 1;
	fi
	local gpio=$1
	local dir=$2
	local dirfile=${SYSFS_GPIO}/gpio${gpio}/direction

	gpio_export ${gpio}
	ret=$?
	if [ $ret -ne 0 ]; then
		return $ret;
	fi;
	if ! [ -w $dirfile ]; then
		echo "gpio ${gpio} not exported"
		return 2;
	fi;
	echo $dir > $dirfile
	ret=$?
	return $ret;
}

imx_gpio_config_dir() {
	if [ "${#}" -ne "3" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi
	local dir=$3
	local gpio=$(imx_gpio $1 $2)
	local ret=$?
	if [ $ret -ne 0 ]; then
		return $ret;
	fi;

	gpio_config_dir ${gpio} $dir
	return $?
}

#
# export one gpio and set it as output
#
gpio_config_out() {
	if [ "${#}" -lt "1" ]
	then
		echo "gpio_config_out: invalid parameter count ${#}";
		return 1;
	fi
	local gpio=$1
	local level=out

	if [ "${#}" -gt "2" ]
	then
		echo "gpio_config_out: invalid parameter count ${#}";
		return 1;
	fi
	if [ "${#}" -eq "2" ]
	then
		if [ "$2" == "low" ]
		then
			level=low;
		elif [ "$2" == "high" ]
		then
			level=high;
		elif [ "$2" -eq "0" ]
		then
			level=low;
		elif [ "$2" -eq "1" ]
		then
			level=high;
		else
			level=$2;
		fi;
	fi
	gpio_export ${gpio};
	gpio_config_dir ${gpio} $level;

	ret=$?
	return $ret
}

#
# export one gpio and set it as output
#
imx_gpio_config_out() {
	if [ "${#}" -lt "2" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi
	if [ "${#}" -gt "3" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$(imx_gpio $1 $2)
	local ret=$?
	if [ "$ret" -eq "0" ]
	then
		if [ "${#}" -eq "3" ]
		then
			gpio_config_out ${gpio} $3;
			ret=$?;		
		else
			gpio_config_out ${gpio};
			ret=$?;		
		fi;
	fi

	return $ret
}

#
# export one gpio and set it as input
#
gpio_config_in() {
	if [ "${#}" -ne "1" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi
	local gpio=$1
	gpio_config_dir ${gpio} in
	ret=$?
	return $ret
}

#
# export one gpio and set it as input
#
imx_gpio_config_in() {
	if [ "${#}" -ne "2" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi

	imx_gpio_config_dir $1 $2 in
	ret=$?
	return $ret
}

#
#
#
gpio_value() {
	if [ "${#}" -ne "1" ]
	then
		echo "gpio_value: invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$1
	local pinval=invalid
	local valfile=${SYSFS_GPIO}/gpio${gpio}/value

	if ! [ -r $valfile ]; then
		echo "gpio ${gpio} not exported"
		return 2;
	fi;

	pinval=$(cat $valfile)
	local ret=$?
	if [ $ret -eq 0 ]; then
		echo ${pinval};
	fi;
	return $ret
}

imx_gpio_value() {
	if [ "${#}" -ne "2" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$(imx_gpio $1 $2)
	local ret=$?
	if [ $ret -ne 0 ]; then
		return $ret;
	fi;
	gpio_value ${gpio}
	return $?
}

#
#
#
gpio_set() {
	if [ "${#}" -ne "2" ]
	then
		echo "_gpio_set: invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$1
	local level=$2
	local valfile=${SYSFS_GPIO}/gpio${gpio}/value
	if ! [ -w $valfile ]; then
		echo "gpio ${gpio} not exported"
		return 0;
	fi;

	echo $level > $valfile
	ret=$?
	if [ $ret -eq 0 ]; then
		cat $valfile;
	fi;
	return $ret
}

imx_gpio_set() {
	if [ "${#}" -ne "3" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$(imx_gpio $1 $2)
	local ret=$?
	if [ $ret -ne 0 ]; then
		return $ret;
	fi;
	gpio_set ${gpio} $3
	return $ret
}

gpio_edge() {
	if [ "${#}" -ne "1" ]
	then
		echo "_gpio_edge: invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$1
	local edgefile=${SYSFS_GPIO}/gpio${gpio}/edge
	local pinedge="unkown"

	if ! [ -r $edgefile ]; then
		echo "gpio ${gpio} not exported"
		return 0;
	fi;

	pinedge=$(cat $edgefile)
	if [ $ret -eq 0 ]; then
		echo ${pin}edge;
	fi;
	return $ret
}

imx_gpio_edge() {
	if [ "${#}" -ne "2" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$(imx_gpio $1 $2)
	local ret=$?
	if [ $ret -ne 0 ]; then
		return $ret;
	fi;

	gpio_edge ${gpio}
	return $?
}

#
# set irq edge
#
gpio_config_edge() {
	if [ "${#}" -ne "2" ]
	then
		echo "_gpio_config_edge: invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$1
	local edgefile=${SYSFS_GPIO}/gpio${gpio}/edge

	if ! [ -w $edgefile ]; then
		echo "gpio ${gpio} not exported"
		return 2;
	fi;

	echo $2 > $edgefile
	ret=$?
	if [ "$ret" -ne "0" ]
	then
		return $ret;
	fi;

	local edge=$(_gpio_edge ${gpio});
	if [ "$edge" != "$2" ]
	then
		echo "setting $2 as edge for gpio ${gpio} failed"
		return 2;
	fi

	return 0
}

imx_gpio_config_edge() {
	if [ "${#}" -ne "3" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$(imx_gpio $1 $2)
	local ret=$?
	if [ "$ret" -ne "0" ]
	then
		return $ret;
	fi;

	gpio_config_edge ${gpio} $3
	return $?
}

#
# set irq edge
#
gpio_config_irq() {
	if [ "${#}" -ne "2" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi
	local gpio=$1
	local edge=$2

	gpio_config_in ${gpio}
	if [ $ret -ne 0 ]
	then
		return $ret;
	fi;

	gpio_config_edge ${gpio} $edge
	return $?
}


imx_gpio_config_irq() {
	if [ "${#}" -ne "3" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi

	imx_gpio_config_in $1 $2
	if [ $ret -ne 0 ]
	then
		return $ret;
	fi;

	imx_gpio_config_edge $1 $2 $3
	if [ "$ret" -ne "0" ]
	then
		return $ret;
	fi;
	ret=$?

	return $ret
}

gpio_poll_irq() {
	if [ "${#}" -ne "2" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi

	local gpiodir=${SYSFS_GPIO}/gpio${gpio}
	if ! [ -d ${gpio}dir ]; then
		echo "gpio ${gpio} not exported"
		return 2;
	fi;

	gpio_irq -g ${gpio} -l $2
	ret=$?
	return $ret
}

imx_gpio_poll_irq() {
	if [ "${#}" -ne "3" ]
	then
		echo "invalid parameter count ${#}";
		return 1;
	fi

	local gpio=$(imx_gpio $1 $2)
	local ret=$?
	if [ $ret -ne 0 ]; then
		return $ret;
	fi;

	gpio_poll_irq ${gpio} $3
	ret=$?
	return $ret
}


