#!/bin/bash
# SPDX-License-Identifier: GPL-3.0-or-later
########################################################################
# file:      pp.sh
#
# Copyright (C) 2016-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Marco Felsch
#
# brief:     Create a patch serie for a given package found under
#            /local_src.
#
# todo:      - Add multiplatform support, implement option -p
#
# note:      23.10.2016 - create file
#
########################################################################

####
# global vars
####
readonly S="$(basename ${0})"
readonly S_DIR="$(dirname ${0})"
P=""
readonly GIT_BASE_OPTIONS="-c core.abbrev=40"
readonly GIT_PATCH_OPTIONS="-N --no-signature"
T_COLORS=""

####
# debugging
####
debug () {
	if [ -n "$DEBUG" ] && [ "$DEBUG" = "on" ]
	then
		$@
	fi
}
debug set -o functrace
debug set -x

####
# func:   pp_print
# brief:  print messags
# param:  $1 - message type (info, err, dbg)
#         $2 - message
# return: ---
####
pp_print () {
	local type="${1}"
	local msg="${2}"

	if [ -z "${T_COLORS}" ]
	then
		# see if it supports colors...
		ncolors=$(tput colors)

		if [ -n "$ncolors" ] && [ ${ncolors} -ge 8 ]
		then
			bold="$(tput bold)"
			underline="$(tput smul)"
			standout="$(tput smso)"
			normal="$(tput sgr0)"
			black="$(tput setaf 0)"
			red="$(tput setaf 1)"
			green="$(tput setaf 2)"
			yellow="$(tput setaf 3)"
			blue="$(tput setaf 4)"
			magenta="$(tput setaf 5)"
			cyan="$(tput setaf 6)"
			white="$(tput setaf 7)"
			T_COLORS="1"
		else
			T_COLORS="0"
		fi
	fi

	case ${type} in
		info)
			if [ "${T_COLORS}" = "0" ]
			then
				echo -e "${S} [INFO]: ${msg}"
			else
				echo -e "${S} ${bold}[INFO]${normal}: ${msg}"
			fi
			;;
		err)
			if [ "${T_COLORS}" = "0" ]
			then
				echo -e "${S} [ERROR]: ${msg}" >&2
			else
				echo -e "${S} ${bold}${red}[ERROR]${normal}: ${msg}" >&2
			fi
			;;
		dbg)
			[ "${DEBUG}" = "on" ] && echo -e "${S} [DEBUG]: ${msg}"
			;;
	esac
}

####
# func:   pp_usage
# brief:  print script usage
# param:  ---
# return: ---
####
pp_usage () {
cat <<- EOF
	Usage: [DEBUG=on] ${S} [OPTION]... [-g] [-c] <PKG-NAME>

	Create a patch serie for a given package found under /local_src.

	Arguments:
	  <PKG-NAME>    package name found under /local_src
	  -g            [optional] patches goes to the global /patches dir
	                or to the platform specific dir <PTXDIST_PLATFORMCONFIGDIR>/patches
	                if not specified.
	  -c            [optional] Commit added patches

	Options:
	  -h print this message
	  -o patch dir (default: <PTXDIST_PLATFORMCONFIGDIR>/patches)
	  -p platform (default: <PTXCONF_PLATFORM>)
	  -t git tag/commit to start patching (default: <PKG>_VERSION)

	Enable debug output via environment variable DEBUG set to "on".
EOF
}

####
# func:   pp_get_pkg_var
# brief:  parse pkg name to ptxconf var
# param:  $1 - pkg name
# return: ptxconf pkg name
####
pp_get_pkg_var () {
	local pkg="${1}"
	local pkg_var=""

	#TODO: fix grep pattern
	if [ "$(${P} print PTX_PACKAGES_SELECTED | grep ${pkg})" ]
	then
		pkg="$(echo ${pkg} | tr '[:lower:]' '[:upper:]')"
		pkg_var="$(echo "${pkg}" | tr '-' '_')"
	else
		pp_print "err" "Your package is not selected. Please select it first and retry the operation."
		return 1
	fi

	echo "${pkg_var}"
	return 0
}

####
# func:   pp_create_add_patches
# brief:  create patches and add them to stagging
# param:  $1 - pkg path
#         $2 - pkg tag/commit
#         $3 - patch dir
#         $4 - bsp-basedir
# return: ---
####
pp_create_add_patches () {
	local pkg_dir="${1}"
	local pkg_tag="${2}"
	local patch_dir="${3}"
	local bsp_dir="${4}"
	local old_dir="$(pwd)"

	#check for existence patches
	if [ -d "${patch_dir}" ] && [ "$(ls -A ${patch_dir})" ]
	then
		pp_print "info" "Deleting old patches..."
		cd ${bsp_dir}
		rm -f ${patch_dir}/*.patch &> /dev/null
		[ "${?}" -ne "0" ] && \
			pp_print "err" "Failure while removing old patches from dir" && \
			return 1
		git rm -f ${patch_dir}/*.patch &> /dev/null
# do not check, in case no patch exists yet
#		[ "${?}" -ne "0" ] && \
#			pp_print "err" "Failure while removing old patches from git" && \
#			return 1
	fi

	#check if tag/commit exists
	cd ${pkg_dir}
	git show ${pkg_tag} &> /dev/null
	if [ "${?}" -ne "0" ]
	then
		pp_print "err" "Tag or Commit does not exits!"
		return 1
	fi

	#create patches and series file
	pp_print "info" "Creating patches since tag/commit: ${pkg_tag}"
	pp_print "dbg" "GIT_BASE_OPTIONS:\t${GIT_BASE_OPTIONS}"
	pp_print "dbg" "GIT_PATCH_OPTIONS:\t${GIT_PATCH_OPTIONS}"
	pp_print "dbg" "git ${GIT_BASE_OPTIONS} format-patch -o ${patch_dir} ${GIT_PATCH_OPTIONS} ${pkg_tag}"
	git ${GIT_BASE_OPTIONS} format-patch -o ${patch_dir} ${GIT_PATCH_OPTIONS} ${pkg_tag} &> /dev/null
	[ "${?}" -ne "0" ] && \
		pp_print "err" "Failure while creating patches" && \
		return 1
	cd ${patch_dir}
	ls -1 *.patch > series

	#add patches
	cd ${bsp_dir}
	git add ${patch_dir}/*.patch &> /dev/null
	[ "${?}" -ne "0" ] && \
		pp_print "err" "Failure while adding patches" && \
		return 1
	git add ${patch_dir}/series &> /dev/null
	[ "${?}" -ne "0" ] && \
		pp_print "err" "Failure while adding series" && \
		return 1

	cd ${old_dir}
	pp_print "info" "All patches are added."
	return 0
}

####
# script start
####
main () {
	# pkg vars
	local pkg_name=""
	local pkg_path=""
	local pkg_ver=""
	local pkg_patch_dir=""
	local pkg_tag=""

	local ptx_pkg_var=""
	local ptx_pkg_var_val=""

	local workpath=""
	local local_src_path=""
	local platform=""
	local commit=""
	local global=""

	pp_print "info" "${S} started..."
	pp_print "dbg" "Script:\t\t${S}"
	pp_print "dbg" "Script dir:\t${S_DIR}"
	pp_print "dbg" "argc ${#}"
	#check existence of ./p link to ptxdist
	if [ -L "${S_DIR}/../p" ]
	then
		P="${S_DIR}/../p"
	else
		pp_print "err" "No p-symlink found! Please config your platform first."
		pp_usage
		exit 1
	fi

	#set path vars
	workpath="$(${P} print PTXDIST_WORKSPACE)"
	local_src_path="${workpath}/local_src"

	pp_print "dbg" "checking input args ..."

	#check input args
	[ ${#} -lt 1 ] && pp_usage && exit 1
	[ ${#} -gt 4 ] && pp_usage && exit 1
	[ -z "${1}" ] && pp_usage && exit 1
	#TODO: need to improve this
	#[ -d "${local_src_path}/${1}" ] || \
	#	[ -L "${local_src_path}/${1}" ] || \
	#	pp_usage && exit 1

	eval ${1+"pkg_name=\${$#"\}}

	pp_print "dbg" "2nd: getopt ..."
	#check options
	while getopts "cgho:p:t:" opt
	do
		pp_print "dbg" "getopt for ${opt} ..."
		case ${opt} in
		c)
			commit=1
			;;
		g)
			global=1
			;;
		h)
			pp_usage
			exit 0
			;;
		o)
			pkg_patch_dir="${OPTARG}"
			;;
		p)
			platform="${OPTARG}"
			;;
		t)
			pkg_tag="${OPTARG}"
			pp_print "dbg" "using tag ${pkg_tag}"
			;;
		\?)
			pp_print "err" "Invalid option: -${OPTARG}"
			pp_print "err" "Type ${S} -h for usage"
			exit 1
			;;
		:)
			pp_print "err" "Option -${OPTARG} requires an argument."
			pp_print "err" "Type ${S} -h for usage"
			exit 1
			;;
		esac
	done

	pp_print "dbg" "pkg_name:\t${pkg_name}"
	ptx_pkg_var="$(pp_get_pkg_var ${pkg_name})"
	[ ${?} -ne 0 ] && exit 1
# WTF?
#	shift
	ptx_pkg_var_val="$(${P} print ${ptx_pkg_var})"

	#set default vars
	[ -z "${platform}" ] && platform="$(${P} print PTXCONF_PLATFORM)"
	pkg_ver="$(${P} print ${ptx_pkg_var}_VERSION)"
	[ -z "${pkg_tag}" ] && pkg_tag="${pkg_ver}"
	if [ -z "${pkg_patch_dir}" ]
	then
		if [ -z "${global}" ]
		then
			pkg_patch_dir="$(${P} print PTXDIST_PLATFORMCONFIGDIR)/patches/${ptx_pkg_var_val}"
		else
			pkg_patch_dir="${workpath}/patches/${ptx_pkg_var_val}"
		fi
	fi
	pkg_path="${local_src_path}/${pkg_name}.${platform}"

	pp_print "dbg" "platform:\t${platform}"
	pp_print "dbg" "pkg_name:\t${pkg_name}"
	pp_print "dbg" "ptx_pkg_var:\t${ptx_pkg_var}"
	pp_print "dbg" "pkg_ver:\t${pkg_ver}"
	pp_print "dbg" "pkg_tag:\t${pkg_tag}"
	pp_print "dbg" "pkg_patch_dir:\t${pkg_patch_dir}"
	pp_print "dbg" "pkg_path:\t${pkg_path}"


	#patching
	pp_create_add_patches ${pkg_path} ${pkg_tag} ${pkg_patch_dir} ${workpath}
	[ "${?}" -ne "0" ] && exit 1
	#commit
	if [ ! -z "${commit}" ]
	then
		if [ -z "${global}" ]
		then
			git commit -m "$(echo ${platform} | tr '[:upper:]' '[:lower:]'): ${pkg_name}: update patch series" &> /dev/null
			[ "${?}" -ne "0" ] && \
				pp_print "err" "Failure while committing patches" && \
				exit 1
		else
			git commit -m "${pkg_name}: update patch series" &> /dev/null
			[ "${?}" -ne "0" ] && \
				pp_print "err" "Failure while committing patches" && \
				exit 1
		fi
		pp_print "info" "All patches are commited."
	fi

	exit 0
}

main ${@}

