#!/bin/bash
# SPDX-License-Identifier: GPL-2.0-or-later
#
# File:        release.sh
#
# Copyright (C) 2014-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel
#
# Description: A utility script that builds an packages the BSP.
#
#              Use release.sh to perform a complete build and create archives of
#              images, source tarballs, dev packages and BSP
#
###############################################################################

# DEBUGGING
# set -e 
set -C # noclobber

readonly PROGRAM="$(basename ${0})"
readonly SCRIPT_LIBDIR="$(dirname ${0})"

# needed for do_query_release_type
PLATFORM_RELEASE_STAMP=
PLATFORM_VERSION_PART=
PLATFORM_PLATFORM_PART=

source "${SCRIPT_LIBDIR}/tools_lib_ptxdist.sh"
source "${SCRIPT_LIBDIR}/tools_lib_git.sh"

readonly TARBALLDIR=sources
readonly DEVPKGDIR=devpkg
readonly DOCUMENTATIONDIR=Documentation/latex

# RETURN VALUES/EXIT STATUS CODES
readonly E_BAD_OPTION=254
readonly E_UNKNOWN=255

function usage () {
    echo "Usage is as follows:"
    echo
    echo "$PROGRAM <--usage|--help|-?>"
    echo "    Prints this usage output and exits."
    echo "$PROGRAM" [--stamp <name>] [--use-git] [--debug] [--license]
    echo "    compile everything and create archives if under git."
    echo " --stamp <name>"
    echo "    compile everything and create archives from <stamp> if under git."
    echo "--use-git"
    echo "    use git for patching instead of quilt (needed for binary patches)."
    echo "--license"
    echo "    extract license info."
    echo "--debug"
    echo "    omit cleaning before collecting data."
    echo
}

#
# $1: base directory
# $2: [optional] git release name
# return 0 on success
#
function extract_license_info () {
	local MYPWD=${1}
	local ret="42"

	if [ -z ${2} ]; then
		${MYPWD}/tools/license.sh
		ret=$?
	else
		${MYPWD}/tools/license.sh --stamp ${2}
		ret=$?
	fi

	if [ "${ret}" -ne "0" ]; then
		echo "license extraction failed, see log.";
		return $E_UNKNOWN;
	fi

	return 0
}

#
# $1: exact ptxdist call
# $2: tarball export dir
# $3: not zero if ptxdist is able to print internal vars
# return 0 on success
#
function export_src_packages () {
	local PTX_PTXDIST=$1
	local THE_TARBALLDIR=$2
	local PTX_CAN_PRINT=$3

	mkdir -p ${THE_TARBALLDIR}
	[ "$?" -ne "0" ] && echo "error: creating ${THE_TARBALLDIR}" && return $E_UNKNOWN

	${PTX_PTXDIST} export_src ${THE_TARBALLDIR}
	if [ $? -ne 0 ]; then
		echo "source package export failed, see log.";
		return $E_UNKNOWN;
	fi

	if [ "${PTX_CAN_PRINT}" -ne "0" ]; then
		${PTX_PTXDIST} print PACKAGES-y >> ${THE_TARBALLDIR}/source.list
		if [ $? -ne 0 ]; then
			echo "package list export failed, see log.";
			return $E_UNKNOWN;
		fi

		${PTX_PTXDIST} print HOST_PACKAGES-y >> ${THE_TARBALLDIR}/source.list
		if [ $? -ne 0 ]; then
			echo "host package list export failed, see log.";
			exit $E_UNKNOWN;
		fi

		${PTX_PTXDIST} print CROSS_PACKAGES-y >> ${THE_TARBALLDIR}/source.list
		if [ $? -ne 0 ]; then
			echo "cross package list export failed, see log.";
			exit $E_UNKNOWN;
		fi
	fi

	return 0
}

function main () {
	local PATCH_OPT=
	local MYPWD=
	local DO_DEBUG="no"
	local HAVE_GIT="0"
	local EXTRACT_LICENSES="0"
	local ret=0
	local PTXDIST_CAN_PRINT="0"
	local CONFIG_FILE_VERSION=""
	local PLATFORM_FILE_VERSION=""
	local PLATFORM
	local PLATFORM_NAME

	# Process command-line arguments.
	while test $# -gt 0; do
	    case $1 in
		--stamp )
		    shift
		    STAMP="$1"
		    shift
		    ;;

		--use-git )
		    shift
		    PATCH_OPT="--git"
		    ;;

		--license )
		    shift
		    EXTRACT_LICENSES="1"
		    ;;

		--debug )
		    shift
		    DO_DEBUG="yes"
		    ;;

		-? | --usage | --help )
		    usage
		    exit
		    ;;

		-* )
		    echo "Unrecognized option: $1" >&2
		    usage
		    exit $E_BAD_OPTION
		    ;;

		* )
		    break
		    ;;
	    esac
	done

	check_ptxdist

	CONFIG_FILE_VERSION=$(get_ptxconfig_version)
	PLATFORM_FILE_VERSION=$(get_platformconfig_version)

	if [ "x${CONFIG_FILE_VERSION}" != "x${PLATFORM_FILE_VERSION}" ]; then
		echo "platform and config versions differ! Please correct.";
		exit $E_UNKNOWN;
	fi

	check_ptxdist_version ${CONFIG_FILE_VERSION}

	PLATFORM_NAME=$(get_platform_name)
	echo "using $PLATFORM_NAME"

	PLATFORM=platform-${PLATFORM_NAME}
	echo "using $PLATFORM"

	PTXDIST=ptxdist-${CONFIG_FILE_VERSION}

	STAMPFILE=${PLATFORM_NAME}-stamp
	MYPWD=$(${PTXDIST} print PTXDIST_WORKSPACE)

	[ check_git ] && HAVE_GIT="1"

	ptxdist_has_working_print ${CONFIG_FILE_VERSION}
	[ "$?" -eq "0" ] && PTXDIST_CAN_PRINT="1"

	do_query_release_type ${CONFIG_FILE_VERSION} ${PLATFORM} ${STAMP}
	[ "${?}" -ne "0" ] && return ${E_UNKNOWN}

	readonly PLATFORM_PKG_SRC_ARCHIVE=${PLATFORM}/dist/${PLATFORM_PLATFORM_PART}.Packages.SRC.${PLATFORM_VERSION_PART}.tar.gz
	readonly PLATFORM_PKG_BIN_ARCHIVE=${PLATFORM}/dist/${PLATFORM_PLATFORM_PART}.Packages.BIN.${PLATFORM_VERSION_PART}.tar.gz
	readonly PLATFORM_BSP_SRC_NAME=${PLATFORM_PLATFORM_PART}.SRC.${PLATFORM_VERSION_PART}
	readonly PLATFORM_BSP_SRC_ARCHIVE=${PLATFORM}/dist/${PLATFORM_PLATFORM_PART}.SRC.${PLATFORM_VERSION_PART}.tar
	readonly PLATFORM_BSP_BIN_ARCHIVE=${PLATFORM}/dist/${PLATFORM_PLATFORM_PART}.BIN.${PLATFORM_VERSION_PART}.tar.gz
	readonly PLATFORM_BSP_MD5FILE=${PLATFORM_PLATFORM_PART}.md5sum.${PLATFORM_VERSION_PART}

	# clean previous build if needed
	rm -rf ${PLATFORM}/${TARBALLDIR}
	rm -rf ${PLATFORM}/dist
	rm -rf ${DOCUMENTATIONDIR}
	rm -rf ${PLATFORM}/${DEVPKGDIR}
	rm -f ${PLATFORM}/images/${STAMPFILE}
	rm -f ${PLATFORM}/${STAMPFILE}

	if [ "${DO_DEBUG}" != "yes" ]; then
		echo "DO_DEBUG: ${DO_DEBUG}"
		${PTXDIST} --force clean
	fi

	if [ "${HAVE_GIT}" -ne "0" ]; then
		git log -1 > ${PLATFORM}/${STAMPFILE}
	fi

	${PTXDIST} ${PATCH_OPT} images
	ret=$?
	if [ $ret -ne 0 ]; then
		echo "build error, see log.";
		exit $E_UNKNOWN;
	fi

	if [ "${EXTRACT_LICENSES}" -ne "0" ]; then
		extract_license_info ${MYPWD} ${STAMP}
		[ "$?" -ne "0" ] && exit $E_UNKNOWN
	fi

	export_src_packages ${PTXDIST} ${PLATFORM}/${TARBALLDIR} ${PTXDIST_CAN_PRINT}
	[ "$?" -ne "0" ] && exit $E_UNKNOWN

	mkdir -p ${PLATFORM}/dist
	! [ -d ${PLATFORM}/dist ] && echo "dist dir creation failed, see log." && exit $E_UNKNOWN;

	ptxdist_has_doc_generation ${CONFIG_FILE_VERSION}
	if [ "${?}" -eq "0" ]; then
		${PTXDIST} docs-latex
		if [ $? -ne 0 ]; then
			echo "doc generation failed, see log.";
			exit $E_UNKNOWN;
		fi

		if [ -d ${DOCUMENTATIONDIR} ]; then
			cp ${DOCUMENTATIONDIR}/*Quickstart.pdf ${PLATFORM}/dist
		else
			echo "documentation not found"
		fi
	fi

	if [ -f ${PLATFORM}/${STAMPFILE} ]; then
		cp ${PLATFORM}/${STAMPFILE} ${PLATFORM}/images
		cp ${PLATFORM}/${STAMPFILE} ${PLATFORM}/${TARBALLDIR};
		cp ${PLATFORM}/${STAMPFILE} ${PLATFORM}/dist;
		cp README*.${PLATFORM_NAME} ${PLATFORM}/dist;
		cp README*.${PLATFORM_NAME} ${PLATFORM}/images;
	else
		echo "${PLATFORM}/${STAMPFILE} not existing ...";
	fi

	tar -cvzf ${PLATFORM_PKG_SRC_ARCHIVE} -C ${PLATFORM} ${TARBALLDIR}
	ret=$?
	if [ $ret -ne 0 ]; then
		echo "${TARBALLDIR} archiving failed, see log.";
		exit $E_UNKNOWN;
	fi

	${MYPWD}/tools/collect_hashes.sh --directory ${PLATFORM}/images > ${PLATFORM}/dist/${PLATFORM_BSP_MD5FILE}
	ret=$?
	if [ $ret -ne 0 ]; then
		echo "image hashing failed.";
		exit $E_UNKNOWN;
	fi

	cp ${PLATFORM}/dist/${PLATFORM_BSP_MD5FILE} ${PLATFORM}/images
	tar -cvzf ${PLATFORM_BSP_BIN_ARCHIVE} -C ${PLATFORM}/images .
	ret=$?
	if [ $ret -ne 0 ]; then
		echo "image archiving failed, see log.";
		exit $E_UNKNOWN;
	fi


	if [ "${HAVE_GIT}" -ne "0" ]; then
		echo "create archive ${PLATFORM_BSP_SRC_ARCHIVE} from ${RELEASE_STAMP} ...";
		$MYPWD/tools/git-archive-all.sh --format tar.gz --prefix ${PLATFORM_BSP_SRC_NAME}/ --commit ${RELEASE_STAMP} ${MYPWD}/${PLATFORM_BSP_SRC_ARCHIVE};
		ret=$?
		if [ $ret -ne 0 ]; then
			echo "git archiving failed, see log.";
			exit $E_UNKNOWN;
		fi
	fi

	echo "collect packages ...";
	mkdir -p ${PLATFORM}/${DEVPKGDIR}
	echo "ipk ...";
	cp ${PLATFORM}/packages/*.ipk ${PLATFORM}/${DEVPKGDIR}

	# copy devpackages only if enabled
	${PTXDIST} print PTXCONF_PROJECT_CREATE_DEVPKGS >/dev/null 2>&1
	ret=$?
	if [ $ret == 0 ]; then
		if [ -f ${PLATFORM}/${STAMPFILE} ]; then
			cp ${PLATFORM}/${STAMPFILE} ${PLATFORM}/${DEVPKGDIR};
		fi
		cp ${PLATFORM}/packages/*-dev.tar.gz ${PLATFORM}/${DEVPKGDIR};
		rm -rf ${PLATFORM}/${DEVPKGDIR}/host-*;
		rm -rf ${PLATFORM}/${DEVPKGDIR}/*opkg*;
		echo "archive packages ...";
	else
		echo "no DEV packages available.";
	fi

	echo "archive packages ...";
	tar -cvzf ${PLATFORM_PKG_BIN_ARCHIVE} -C ${PLATFORM} ${DEVPKGDIR}

	exit 0
}

main $@

